<?php
/**
 * Laminas Form Label View Helper
 *
 * Copyright (C) 2019 Leipzig University Library
 *
 * PHP Version 7
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @author   Sebastian Kehr <kehr@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace fid\View\Helper\Root;

use Laminas\Form\Element;
use Laminas\Form\ElementInterface;
use Laminas\Form\View\Helper\FormLabel as Base;

/**
 * Improved Laminas Form Label View Helper by adding attributes
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @author   Sebastian Kehr <kehr@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 */
class FormLabel extends Base
{
    /**
     * {@inheritdoc}
     *
     * @param null|array|ElementInterface $element Form element
     *
     * @return string
     */
    public function openTag($element = null)
    {
        if ($element instanceof Element) {
            $element->setLabelAttributes(
                [
                    'data-type'     => $element->getAttribute('type'),
                    'data-name'     => $element->getAttribute('name'),
                    'data-required' => $element->getAttribute('required')
                        ? 'true' : 'false'
                ] + $element->getLabelAttributes()
            );
        }

        return parent::openTag($element);
    }
}
