<?php
/**
 * Finc Logger for time measurement
 *
 * PHP version 7
 *
 * Copyright (C) 2023 Leipzig University Library
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @category Finc
 * @package  Error_Logging
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development
 */
namespace finc\Log;

use VuFind\Log\Logger as BaseLogger;

/**
 * This class wraps the BaseLogger class to allow for log verbosity
 *
 * @category VuFind
 * @package  Error_Logging
 * @author   Chris Hallberg <challber@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Site
 */
class PerformanceLogger extends BaseLogger
{
    public static $cache = [];

    /**
     * {@inheritDoc}
     *
     * @param int               $priority Priority
     * @param mixed             $message  Message
     * @param array|Traversable $extra    Extras
     *
     * @return Logger
     */
    public function log($priority, $message, $extra = [])
    {
        // ignore complex messages
        if (!is_array($message)) {
            if (!isset($extra['qtime'])) {
                // cache until query was finished
                array_push(self::$cache, $message);
            } else {
                // flush to log
                $message = ';' . implode(" ", self::$cache) . ';' . $extra['qtime'] . ';';
                self::$cache = [];
                return parent::log($priority, $message);
            }
        }
        return $this;
    }

    /**
     * Log time measurement
     *
     * @param string $message log message
     * @param array  $extra   additional values
     *
     * @return $this|PerformanceLogger
     */
    public function performance($message, $extra = [])
    {
        return parent::log(self::DEBUG, $message, $extra);
    }
}
