<?php
/**
 * View helper to render a portion of an array.
 *
 * PHP version 5
 *
 * Copyright (C) Villanova University 2010.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace finc\View\Helper\Root;

use Laminas\View\Helper\AbstractHelper;

/**
 * View helper to render a portion of an array.
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
class EnhancedRenderArray extends AbstractHelper
{
    /**
     * Render a portion of an array.
     *
     * @param string $tpl1   A template for displaying each row as read-only value.
     * This should include %%LABEL%% and %%VALUE%% placeholders
     * @param string $tpl2   A template for displaying each row as input field.
     * A 2-dim-array including arrays for every line with label and autocomplete
     * This should include %%KEY&&, %%LABEL%%, %%VALUE%%, %%AUTOCOMPLETE%%
     * @param array  $arr    An associative array of possible values to display
     * @param array  $arrays A label => profile key associative array specifying
     * which rows of $arr to display
     *
     * @return string
     */
    public function __invoke($tpl1, $tpl2, $arr, $arrays)
    {
        $html = '';
        foreach ($arrays as $key => $row) {
            $html .= str_replace(
                ['%%KEY%%', '%%LABEL%%', '%%VALUE%%', '%%AUTOCOMPLETE%%'],
                [$key, $row['label'], $this->view->escapeHtml(($arr[$key] ?? '')), $row['autocomplete']],
                (isset($arr['disabled'][$key]) && ($arr['disabled'][$key] === true) ? $tpl1 : $tpl2)
            );
        }
        return $html;
    }
}
