<?php
namespace Selenium\finc;

use Lmc\Steward\ConfigProvider;
use Lmc\Steward\Test\AbstractTestCase;

class FincBase extends AbstractTestCase
{
    public static $port = '';
    public static $host = 'https://staging.finc.info/vufind2/de_zwi2';

    protected $user = false;

    /**
     * @before
     */
    public function initBaseUrl()
    {
        // Set base url according to environment
        $env = ConfigProvider::getInstance()->env;
        switch ($env) {
            case 'staging':
                self::$host = 'https://staging.finc.info/vufind2/local';
                break;
            case 'local':
                self::$host = 'http://host.docker.internal';
                break;
            default:
                // assume alpha with syntax [instance]/[issue number]
                self::$host = 'https://alpha.finc.info/vufind2/' . $env;
        }

        $this->debug('Base URL set to "%s"', self::$host);

        if (ConfigProvider::getInstance()->env === 'production') {
            $this->warn('The tests are run against production, so be careful!');
        }
    }

    protected function runTestChangeProfile($user, $password)
    {
        $this->wd->get(self::$host . self::$port);
        $this->logout();
        $this->changeLanguage();
        $this->user = $user;
        $this->login("$this->user", $password);
        $this->findByXpath('//*[@id="content"]/div[1]/a')->click();
        $this->waitForName('accountForm');
        $inputFirstname = $this->findById('firstname');
        $oldFirstname = $inputFirstname->getAttribute('value');
        $this->log("Old Vorname: $oldFirstname");
        // $this->wd->takeScreenshot('logs/old.png');
        $inputFirstname->clear();
        $newFirstname = time();
        $inputFirstname->sendKeys($newFirstname);
        // $this->wd->takeScreenshot('logs/change.png');
        $this->findById('submitAccount')->click();
        $this->waitForClass('alert-success');
        // $this->wd->takeScreenshot('logs/success.png');
        $changedValue = $this->findByXpath('//*[@id="content"]/div[1]/table/tbody/tr[1]/td')->getText();
        // restore old name
        $this->findByXpath('//*[@id="content"]/div[1]/a')->click();
        $this->waitForName('accountForm');
        $this->findById('firstname')->clear();
        $this->findById('firstname')->sendKeys($oldFirstname);
        $this->findById('submitAccount')->click();
        // $this->wd->takeScreenshot('logs/restored.png');

        $this->assertStringContainsStringIgnoringCase(
            $newFirstname,
            $changedValue,
            "Vorname could not be updated."
        );
    }

    #region helper methods
    protected function changeLanguage($lang = 'de')
    {
        $this->wd->get(self::$host . self::$port);
        $this->wd->executeScript("document.langForm.mylang.value='" . $lang . "';document.langForm.submit();");
        $this->waitForPartialLinkText('Konto');
    }

    protected function setFullsreen()
    {
        $this->wd->manage()->window()->maximize();
    }

    protected function logout()
    {
        $this->wd->get(self::$host . self::$port . '/MyResearch/Logout');
    }

    protected function login($user, $password): bool
    {
        $this->wd->get(self::$host . self::$port . '/MyResearch/UserLogin');
        $this->findById('login_ILS_username')->sendKeys($user);
        $this->findById('login_ILS_password')->sendKeys($password);
        $this->findByName('processLogin')->click();
        $this->waitForClass('myresearch-menu');
        return true;
    }
    #endregion
}
