<?php
/**
 * Copyright (C) 2023 Leipzig University Library
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Run for local with via:
 * docker exec -it selenium_php_1 sh -c "./vendor/bin/steward run local firefox --group performance-static --server-url http://selenium:4444/wd/hub -vvv"
 *
 * Run for alpha with via:
 * docker exec -it selenium_php_1 sh -c "./vendor/bin/steward run de_15/12345 firefox --group performance-static --server-url http://selenium:4444/wd/hub -vvv"
 *
 * @category Finc
 * @package  Test
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development
 */
namespace Selenium\performance;

use Lmc\Steward\ConfigProvider;
use Selenium\finc\FincBase;

class SearchBaseTest extends FincBase
{
    public static $words = [
        'Antenne',
        'Bach',
        'Cello',
        'Dach',
        'Eiche',
        'Fach',
        'Gans',
        'Haus',
        'Igel',
        'Jagd',
        'Kabel',
        'Lampe',
        'Maus',
        'Nacht',
        'Ohr',
        'Pferd',
        'Quelle',
        'Rabe',
        'Sonne',
        'Tisch',
        'Uhr',
        'Vogel',
        'Wald',
        'Xylophon',
        'Yoga',
        'Zebra'
    ];

    const STATIC_SEARCH_TERM = 'Medienkritik';

    /**
     * @before
     */
    public function initBaseUrl()
    {
        // Set base url according to environment
        $env = ConfigProvider::getInstance()->env;
        switch ($env) {
            case 'staging':
                self::$host = 'https://staging.finc.info/vufind2/local';
                break;
            case 'local':
                self::$host = 'http://host.docker.internal';
                break;
            default:
                // assume alpha with syntax [instance]/[issue number]
                self::$host = 'https://alpha.finc.info/vufind2/' . $env;
        }

        $this->debug('Base URL set to "%s"', self::$host);

        if (ConfigProvider::getInstance()->env === 'production') {
            $this->warn('The tests are run against production, so be careful!');
        }
    }

    #region helper methods
    protected function getBasePath() : string
    {
        return self::$host . self::$port . '/Search/Results?lookfor=';
    }

    protected function getSearchTerm(bool $isDynamicSearch = true) : string
    {
        if ($isDynamicSearch) {
            $rand_keys = array_rand(self::$words, 2);
            $searchTerm = self::$words[$rand_keys[0]] . " " . self::$words[$rand_keys[1]];
        } else {
            $searchTerm = self::STATIC_SEARCH_TERM;
        }

        return urlencode($searchTerm);
    }

    #endregion
}
