<?php
/**
 * Copyright (C) 2023 Leipzig University Library
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Run for local with via:
 * docker exec -it selenium_php_1 sh -c "./vendor/bin/steward run local firefox --group performance-static --server-url http://selenium:4444/wd/hub -vvv"
 *
 * Run for alpha with via:
 * docker exec -it selenium_php_1 sh -c "./vendor/bin/steward run de_15/12345 firefox --group performance-static --server-url http://selenium:4444/wd/hub -vvv"
 *
 * @category Finc
 * @package  Test
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development
 */
namespace Selenium\performance;

use Lmc\Steward\ConfigProvider;

/**
 * @group performance
 * @group performance-static
 */
class SearchStaticTest extends SearchBaseTest
{
    /**
     * @group performance
     * @group performance-static
     */
    public function testStaticSearches()
    {
        $start = floor(microtime(true) * 1000);
        for ($i = 1; $i <= 100; $i++) {
            $this->debug("Executing test no. $i");
            $this->wd->get($this->getBasePath() . $this->getSearchTerm(false));
            $this->waitForClass('result-body');
        }
        $end = floor(microtime(true) * 1000);
        $durationText = "Duration of static searches: " . ((intval($end)) - (intval($start)) . " ms");
        $this->debug($durationText);
    }
}
